@@ -261,7 +261,7 @@ set_macro_sequence (const char *regexp)
 
   msg = re_compile_pattern (regexp, strlen (regexp), &macro_sequence_buf);
   if (msg != NULL)
-    m4_failure (0, "--warn-macro-sequence: bad regular expression `%s': %s",
+    m4_failure (0, _("--warn-macro-sequence: bad regular expression `%s': %s"),
                 regexp, msg);
   re_set_registers (&macro_sequence_buf, &macro_sequence_regs,
                     macro_sequence_regs.num_regs,
@@ -317,14 +317,14 @@ define_user_macro (const char *name, const char *text, symbol_lookup mode)
               tmp = defn[offset];
               defn[offset] = '\0';
               M4ERROR ((warning_status, 0,
-                        "Warning: definition of `%s' contains sequence `%s'",
+                        _("Warning: definition of `%s' contains sequence `%s'"),
                         name, defn + macro_sequence_regs.start[0]));
               defn[offset] = tmp;
             }
         }
       if (offset == -2)
         M4ERROR ((warning_status, 0,
-                  "error checking --warn-macro-sequence for macro `%s'",
+                  _("error checking --warn-macro-sequence for macro `%s'"),
                   name));
     }
 }
@@ -385,13 +385,13 @@ bad_argc (token_data *name, int argc, int min, int max)
     {
       if (!suppress_warnings)
         M4ERROR ((warning_status, 0,
-                  "Warning: too few arguments to builtin `%s'",
+                  _("Warning: too few arguments to builtin `%s'"),
                   TOKEN_DATA_TEXT (name)));
       isbad = true;
     }
   else if (max > 0 && argc > max && !suppress_warnings)
     M4ERROR ((warning_status, 0,
-              "Warning: excess arguments to builtin `%s' ignored",
+              _("Warning: excess arguments to builtin `%s' ignored"),
               TOKEN_DATA_TEXT (name)));
 
   return isbad;
@@ -412,7 +412,7 @@ numeric_arg (token_data *macro, const char *arg, int *valuep)
     {
       *valuep = 0;
       M4ERROR ((warning_status, 0,
-                "empty string treated as 0 in builtin `%s'",
+                _("empty string treated as 0 in builtin `%s'"),
                 TOKEN_DATA_TEXT (macro)));
     }
   else
@@ -422,17 +422,17 @@ numeric_arg (token_data *macro, const char *arg, int *valuep)
       if (*endp != '\0')
         {
           M4ERROR ((warning_status, 0,
-                    "non-numeric argument to builtin `%s'",
+                    _("non-numeric argument to builtin `%s'"),
                     TOKEN_DATA_TEXT (macro)));
           return false;
         }
       if (c_isspace (*arg))
         M4ERROR ((warning_status, 0,
-                  "leading whitespace ignored in builtin `%s'",
+                  _("leading whitespace ignored in builtin `%s'"),
                   TOKEN_DATA_TEXT (macro)));
       else if (errno == ERANGE)
         M4ERROR ((warning_status, 0,
-                  "numeric overflow detected in builtin `%s'",
+                  _("numeric overflow detected in builtin `%s'"),
                   TOKEN_DATA_TEXT (macro)));
     }
   return true;
@@ -549,7 +549,7 @@ define_macro (int argc, token_data **argv, symbol_lookup mode)
   if (TOKEN_DATA_TYPE (argv[1]) != TOKEN_TEXT)
     {
       M4ERROR ((warning_status, 0,
-                "Warning: %s: invalid macro name ignored", ARG (0)));
+                _("Warning: %s: invalid macro name ignored"), ARG (0)));
       return;
     }
 
@@ -749,7 +749,7 @@ m4_dumpdef (struct obstack *obs, int argc, token_data **argv)
             dump_symbol (s, &data);
           else
             M4ERROR ((warning_status, 0,
-                      "undefined macro `%s'", TOKEN_DATA_TEXT (argv[i])));
+                      _("undefined macro `%s'"), TOKEN_DATA_TEXT (argv[i])));
         }
     }
 
@@ -812,7 +812,7 @@ m4_builtin (struct obstack *obs, int argc, token_data **argv)
   if (TOKEN_DATA_TYPE (argv[1]) != TOKEN_TEXT)
     {
       M4ERROR ((warning_status, 0,
-                "Warning: %s: invalid macro name ignored", ARG (0)));
+                _("Warning: %s: invalid macro name ignored"), ARG (0)));
       return;
     }
 
@@ -820,7 +820,7 @@ m4_builtin (struct obstack *obs, int argc, token_data **argv)
   bp = find_builtin_by_name (name);
   if (bp->func == m4_placeholder)
     M4ERROR ((warning_status, 0,
-              "undefined builtin `%s'", name));
+              _("undefined builtin `%s'"), name));
   else
     {
       int i;
@@ -853,7 +853,7 @@ m4_indir (struct obstack *obs, int argc, token_data **argv)
   if (TOKEN_DATA_TYPE (argv[1]) != TOKEN_TEXT)
     {
       M4ERROR ((warning_status, 0,
-                "Warning: %s: invalid macro name ignored", ARG (0)));
+                _("Warning: %s: invalid macro name ignored"), ARG (0)));
       return;
     }
 
@@ -861,7 +861,7 @@ m4_indir (struct obstack *obs, int argc, token_data **argv)
   s = lookup_symbol (name, SYMBOL_LOOKUP);
   if (s == NULL || SYMBOL_TYPE (s) == TOKEN_VOID)
     M4ERROR ((warning_status, 0,
-              "undefined macro `%s'", name));
+              _("undefined macro `%s'"), name));
   else
     {
       int i;
@@ -911,11 +911,11 @@ m4_defn (struct obstack *obs, int argc, token_data **argv)
         case TOKEN_FUNC:
           b = SYMBOL_FUNC (s);
           if (b == m4_placeholder)
-            M4ERROR ((warning_status, 0, "\
-builtin `%s' requested by frozen file is not supported", arg));
+            M4ERROR ((warning_status, 0, _("\
+builtin `%s' requested by frozen file is not supported"), arg));
           else if (argc != 2)
             M4ERROR ((warning_status, 0,
-                      "Warning: cannot concatenate builtin `%s'",
+                      _("Warning: cannot concatenate builtin `%s'"),
                       arg));
           else
             push_macro (b);
@@ -975,7 +975,7 @@ m4_syscmd (struct obstack *obs MAYBE_UNUSED, int argc, token_data **argv)
   else
     {
       if (status == 127 && errno)
-        M4ERROR ((warning_status, errno, "cannot run command `%s'", cmd));
+        M4ERROR ((warning_status, errno, _("cannot run command `%s'"), cmd));
       sysval = status;
     }
 }
@@ -1012,7 +1012,7 @@ m4_esyscmd (struct obstack *obs, int argc, token_data **argv)
                           NULL, false, true, false, &fd);
   if (child == -1)
     {
-      M4ERROR ((warning_status, errno, "cannot run command `%s'", cmd));
+      M4ERROR ((warning_status, errno, _("cannot run command `%s'"), cmd));
       sysval = 127;
       return;
     }
@@ -1025,7 +1025,7 @@ m4_esyscmd (struct obstack *obs, int argc, token_data **argv)
 #endif
   if (pin == NULL)
     {
-      M4ERROR ((warning_status, errno, "cannot run command `%s'", cmd));
+      M4ERROR ((warning_status, errno, _("cannot run command `%s'"), cmd));
       sysval = 127;
       close (fd);
       return;
@@ -1048,7 +1048,7 @@ m4_esyscmd (struct obstack *obs, int argc, token_data **argv)
       obstack_blank_fast (obs, len);
     }
   if (ferror (pin) || fclose (pin))
-    m4_failure (errno, "cannot read pipe");
+    m4_failure (errno, _("cannot read pipe"));
   errno = 0;
   status = wait_subprocess (child, ARG (0), false, true, true, false,
                             &sig_status);
@@ -1060,7 +1060,7 @@ m4_esyscmd (struct obstack *obs, int argc, token_data **argv)
   else
     {
       if (status == 127 && errno)
-        M4ERROR ((warning_status, errno, "cannot run command `%s'", cmd));
+        M4ERROR ((warning_status, errno, _("cannot run command `%s'"), cmd));
       sysval = status;
     }
 }
@@ -1095,7 +1095,7 @@ m4_eval (struct obstack *obs, int argc, token_data **argv)
   if (radix < 1 || radix > (int) strlen (digits))
     {
       M4ERROR ((warning_status, 0,
-                "radix %d in builtin `%s' out of range",
+                _("radix %d in builtin `%s' out of range"),
                 radix, ARG (0)));
       return;
     }
@@ -1105,13 +1105,13 @@ m4_eval (struct obstack *obs, int argc, token_data **argv)
   if (min < 0)
     {
       M4ERROR ((warning_status, 0,
-                "negative width to builtin `%s'", ARG (0)));
+                _("negative width to builtin `%s'"), ARG (0)));
       return;
     }
 
   if (!*ARG (1))
     M4ERROR ((warning_status, 0,
-              "empty string treated as 0 in builtin `%s'", ARG (0)));
+              _("empty string treated as 0 in builtin `%s'"), ARG (0)));
   else if (evaluate (ARG (1), &value))
     return;
 
@@ -1240,7 +1240,7 @@ m4_undivert (struct obstack *obs MAYBE_UNUSED, int argc, token_data **argv)
           insert_diversion (file);
         else if (no_gnu_extensions)
           M4ERROR ((warning_status, 0,
-                    "non-numeric argument to builtin `%s'", ARG (0)));
+                    _("non-numeric argument to builtin `%s'"), ARG (0)));
         else
           {
             fp = m4_path_search (ARG (i), NULL);
@@ -1249,11 +1249,11 @@ m4_undivert (struct obstack *obs MAYBE_UNUSED, int argc, token_data **argv)
                 insert_file (fp);
                 if (fclose (fp) == EOF)
                   M4ERROR ((warning_status, errno,
-                            "error undiverting `%s'", ARG (i)));
+                            _("error undiverting `%s'"), ARG (i)));
               }
             else
               M4ERROR ((warning_status, errno,
-                        "cannot undivert `%s'", ARG (i)));
+                        _("cannot undivert `%s'"), ARG (i)));
           }
       }
 }
@@ -1363,7 +1363,7 @@ include (int argc, token_data **argv, bool silent)
     {
       if (!silent)
         {
-          M4ERROR ((warning_status, errno, "cannot open `%s'", ARG (1)));
+          M4ERROR ((warning_status, errno, _("cannot open `%s'"), ARG (1)));
           retcode = EXIT_FAILURE;
         }
       return;
@@ -1426,7 +1426,7 @@ mkstemp_helper (struct obstack *obs, const char *me, const char *pattern,
   fd = mkstemp (name);
   if (fd < 0)
     {
-      M4ERROR ((0, errno, "%s: cannot create tempfile `%s'", me, pattern));
+      M4ERROR ((0, errno, _("%s: cannot create tempfile `%s'"), me, pattern));
       obstack_free (obs, obstack_finish (obs));
     }
   else
@@ -1461,7 +1461,7 @@ m4_maketemp (struct obstack *obs, int argc, token_data **argv)
       int i;
       int len2;
 
-      M4ERROR ((warning_status, 0, "recommend using mkstemp instead"));
+      M4ERROR ((warning_status, 0, _("recommend using mkstemp instead")));
       for (i = len; i > 1; i--)
         if (str[i - 1] != 'X')
           break;
@@ -1554,7 +1554,7 @@ m4_m4exit (struct obstack *obs MAYBE_UNUSED, int argc, token_data **argv)
   if (exit_code < 0 || exit_code > 255)
     {
       M4ERROR ((warning_status, 0,
-                "exit status out of range: `%d'", exit_code));
+                _("exit status out of range: `%d'"), exit_code));
       exit_code = EXIT_FAILURE;
     }
   /* Change debug stream back to stderr, to force flushing debug stream and
@@ -1678,7 +1678,7 @@ m4_debugmode (struct obstack *obs MAYBE_UNUSED, int argc, token_data **argv)
 
       if (new_debug_level < 0)
         M4ERROR ((warning_status, 0,
-                  "Debugmode: bad debug flags: `%s'", ARG (1)));
+                  _("Debugmode: bad debug flags: `%s'"), ARG (1)));
       else
         {
           switch (change_flag)
@@ -1719,7 +1719,7 @@ m4_debugfile (struct obstack *obs MAYBE_UNUSED, int argc, token_data **argv)
     debug_set_output (NULL);
   else if (!debug_set_output (ARG (1)))
     M4ERROR ((warning_status, errno,
-              "cannot set debug file `%s'", ARG (1)));
+              _("cannot set debug file `%s'"), ARG (1)));
 }
 
 /* This section contains text processing macros: "len", "index",
@@ -1985,8 +1985,8 @@ substitute (struct obstack *obs, const char *victim, const char *repl,
         case '0':
           if (!substitute_warned)
             {
-              M4ERROR ((warning_status, 0, "\
-Warning: \\0 will disappear, use \\& instead in replacements"));
+              M4ERROR ((warning_status, 0, _("\
+Warning: \\0 will disappear, use \\& instead in replacements")));
               substitute_warned = 1;
             }
           FALLTHROUGH;
@@ -2001,7 +2001,7 @@ Warning: \\0 will disappear, use \\& instead in replacements"));
           ind = ch -= '0';
           if (regs->num_regs - 1 <= ind)
             M4ERROR ((warning_status, 0,
-                      "Warning: sub-expression %d not present", ch));
+                      _("Warning: sub-expression %d not present"), ch));
           else if (regs->end[ch] > 0)
             obstack_grow (obs, victim + regs->start[ch],
                           regs->end[ch] - regs->start[ch]);
@@ -2010,7 +2010,7 @@ Warning: \\0 will disappear, use \\& instead in replacements"));
 
         case '\0':
           M4ERROR ((warning_status, 0,
-                    "Warning: trailing \\ ignored in replacement"));
+                    _("Warning: trailing \\ ignored in replacement")));
           return;
 
         default:
@@ -2076,7 +2076,7 @@ m4_regexp (struct obstack *obs, int argc, token_data **argv)
   if (msg != NULL)
     {
       M4ERROR ((warning_status, 0,
-                "bad regular expression: `%s': %s", regexp, msg));
+                _("bad regular expression: `%s': %s"), regexp, msg));
       free_pattern_buffer (&buf, &regs);
       return;
     }
@@ -2088,7 +2088,7 @@ m4_regexp (struct obstack *obs, int argc, token_data **argv)
 
   if (startpos == -2)
     M4ERROR ((warning_status, 0,
-               "error matching regular expression `%s'", regexp));
+              _("error matching regular expression `%s'"), regexp));
   else if (argc == 3)
     shipout_int (obs, startpos);
   else if (startpos >= 0)
@@ -2136,7 +2136,7 @@ m4_patsubst (struct obstack *obs, int argc, token_data **argv)
   if (msg != NULL)
     {
       M4ERROR ((warning_status, 0,
-                "bad regular expression `%s': %s", regexp, msg));
+                _("bad regular expression `%s': %s"), regexp, msg));
       free (buf.buffer);
       return;
     }
@@ -2158,7 +2158,7 @@ m4_patsubst (struct obstack *obs, int argc, token_data **argv)
 
           if (matchpos == -2)
             M4ERROR ((warning_status, 0,
-                      "error matching regular expression `%s'", regexp));
+                      _("error matching regular expression `%s'"), regexp));
           else if (offset < length)
             obstack_grow (obs, victim + offset, length - offset);
           break;
@@ -2205,8 +2205,8 @@ void
 m4_placeholder (struct obstack *obs MAYBE_UNUSED, int argc,
                 token_data **argv)
 {
-  M4ERROR ((warning_status, 0, "\
-builtin `%s' requested by frozen file is not supported", ARG (0)));
+  M4ERROR ((warning_status, 0, _("\
+builtin `%s' requested by frozen file is not supported"), ARG (0)));
 }
 
 /*-------------------------------------------------------------------.
