@@ -1,5 +1,5 @@
 /* Test of <dirent.h> substitute in C++ mode.
-   Copyright (C) 2010-2016 Free Software Foundation, Inc.
+   Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Bruno Haible <bruno@clisp.org>, 2010.  */
 
@@ -24,14 +24,30 @@
 #include "signature.h"
 
 
+#if GNULIB_TEST_OPENDIR
+SIGNATURE_CHECK (GNULIB_NAMESPACE::opendir, DIR *, (const char *));
+#endif
+
+#if GNULIB_TEST_READDIR
+SIGNATURE_CHECK (GNULIB_NAMESPACE::readdir, struct dirent *, (DIR *));
+#endif
+
+#if GNULIB_TEST_REWINDDIR
+SIGNATURE_CHECK (GNULIB_NAMESPACE::rewinddir, void, (DIR *));
+#endif
+
+#if GNULIB_TEST_CLOSEDIR
 SIGNATURE_CHECK (GNULIB_NAMESPACE::closedir, int, (DIR *));
+#endif
+
+#if GNULIB_TEST_DIRFD
+SIGNATURE_CHECK (GNULIB_NAMESPACE::dirfd, int, (DIR *));
+#endif
 
 #if GNULIB_TEST_FDOPENDIR
 SIGNATURE_CHECK (GNULIB_NAMESPACE::fdopendir, DIR *, (int));
 #endif
 
-SIGNATURE_CHECK (GNULIB_NAMESPACE::opendir, DIR *, (const char *));
-
 #if GNULIB_TEST_SCANDIR
 SIGNATURE_CHECK (GNULIB_NAMESPACE::scandir, int,
                  (const char *, struct dirent ***,
